/*
 *  This file is part of the nuvtools package
 *  Copyright (C) 2004 Gavin Hurlbut
 *
 *  nuvtools is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _nuvread_h
#define _nuvread_h

#include "format.h"

typedef struct {
    int             num;
    int             denom;
} fraction;

typedef struct {
    int             fd;
    int             width;
    int             height;
    fraction        fps;
    int             gop_length;
    unsigned int    out_video_4cc;
    unsigned int    video_4cc;
    unsigned int    audio_4cc;
    int             audio_sample_rate_file;
    int             audio_sample_rate;
    int             audio_bits_per_sample;
    int             audio_channels;
    int             audio_compression_ratio;
    int             audio_quality;
    int             audio_bitrate_calc;
    int             audio_bitrate_closest;
    int             video_bitrate;
    int             rtjpeg_quality;
    int             rtjpeg_luma;
    int             rtjpeg_chroma;
    long long int   seektable_offset;
    long long int   kfa_offset;
    long long int   frameCount;
    int             lastVideo;
    int             lastAudio;
    int             verbose;
    int             frameSize;
} nuvFile;

typedef struct {
    rtframeheader   header;
    int             needsEncode;
    int             offset;
    int             length;
    char           *data;
    int             length2;
    char           *data2;
} nuvFrame;

nuvFrame *nuv_new_frame( void );
void nuv_free_frame( nuvFrame *frame );
nuvFile *nuv_init( char *filename, int verbose );
nuvFrame *nuv_read_frame( nuvFile *nv, char *searchList );
void nuv_dump_frame( nuvFile *nv, nuvFrame *frame );
void nuv_hex_dump( nuvFrame *frame );
void nuv_free_file( nuvFile *nv );
nuvFrame *nuv_duplicate_frame( nuvFrame *frame );

#endif


/*
 * vim:ts=4:sw=4:ai:et:si:sts=4
 */
