/*
 *  This file is part of the nuvtools package
 *  Copyright (C) 2004, 2005 Gavin Hurlbut
 *
 *  nuvtools is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*HEADER---------------------------------------------------
* $Id: audReencode.c 105 2005-04-24 03:10:16Z gjhurlbu $
*
* Copyright 2004, 2005 Gavin Hurlbut
* All rights reserved
*
*----------------------------------------------------------
*/

#include "environment.h"
#include <stdlib.h>
#include "format.h"
#include "nuvread.h"
#include "queue.h"
#include "linked_list.h"
#include "avcodec.h"
#include "types.h"
#include "logging.h"

/* CVS generated ID string */
static char ident[] _UNUSED_ = 
    "$Id: audReencode.c 105 2005-04-24 03:10:16Z gjhurlbu $";

extern QueueObject_t *AudioInQueue;
extern QueueObject_t *WritingQueue;
extern bool GlobalAbort;

static nuvFile        *nv;
static nuvFrame       *frame;
static short int      *pcmL, *pcmR;
static nuvFrame       *frameAudio;

nuvFrame *decodeMPx( nuvFile *nv, nuvFrame *frameIn );
nuvFrame *decodePCM( nuvFile *nv, nuvFrame *frameIn );
int findDecode( fourccDef_t *fourCCTable, int count, unsigned char frameType,
                unsigned int fourcc );
void initAudioEncode( nuvFile *nv );
void decodeAudioInit( nuvFile *nv );
void decodeAudioClose( void );
nuvFrame *encodeAudio( nuvFile *nv, int interleaved, short int *pcmL,
                       short int *pcmR, int pcmSize );
void closeAudio( void );

static fourccDef_t fourCC[] = {
    { '0', { .text = "RAWA" }, decodePCM },
    { '3', { .text = "LAME" }, decodeMPx }
};
static int fourCC_count = NELEMENTS(fourCC);


void *AudioReencodeThread( void *arg )
{
    int             done;
    int             decodeIndex;
    int             pcmSize;
    int             count;
    int             LastSync;
    int             interleaved;

    done = FALSE;
    pcmSize = 0;
    pcmL = NULL;
    pcmR = NULL;
    LastSync = -1;

    while( !done ) {
        frame = (nuvFrame *)QueueDequeueItem( AudioInQueue, -1 );
        if( !frame || GlobalAbort ) {
            return( NULL );
        }

        switch( frame->header.frametype ) {
        case 'I':
            nv = (nuvFile *)malloc( sizeof(nuvFile) );
            if( !nv ) {
                LogPrint( LOG_CRIT, "Couldn't copy the file info in "
                                    "audReencode\n" );
                return( NULL );
            }
            memcpy( nv, frame->data, frame->length );

            initAudioEncode(nv);
            decodeAudioInit(nv);

            QueueEnqueueItem( WritingQueue, (QueueItem_t)frame );
            break;
        case 'E':
            QueueEnqueueItem( WritingQueue, (QueueItem_t)frame );
            done = TRUE;
            break;
        case 'A':
            if( LastSync == -1 ) {
                LastSync = frame->header.timecode;
            }
            decodeIndex = findDecode( fourCC, fourCC_count, 
                                      frame->header.comptype, nv->audio_4cc );
            if( decodeIndex != -1 ) {
                frame = fourCC[decodeIndex].decode(nv, frame);
                if( frame ) {
                    switch( frame->needsEncode ) {
                    case 1:
                        /* interleaved data -> use pcmL only */
                        pcmL = realloc( pcmL, pcmSize + frame->length );
                        memcpy( &((char *)pcmL)[pcmSize], frame->data, 
                                frame->length );
                        pcmSize += frame->length;
                        interleaved = TRUE;
                        break;
                    case 2:
                        /* Non-interleaved data -> pcmL, then pcmR */
                        count = frame->length;
                        pcmL = realloc( pcmL, pcmSize + count );
                        pcmR = realloc( pcmR, pcmSize + count );

                        memcpy( &((char *)pcmL)[pcmSize], frame->data, count );
                        memcpy( &((char *)pcmR)[pcmSize], frame->data2, count );
                        pcmSize += count;
                        interleaved = FALSE;
                        break;
                    default:
                        /* WTF is this crap? */
                        break;
                    }

                    nuv_free_frame( frame );
                }
            }
            break;
        case 'S':
            if( pcmSize != 0 ) {
                frameAudio = encodeAudio( nv, interleaved, pcmL, pcmR, 
                                          pcmSize );
                if( frameAudio ) {
                    frameAudio->header.timecode = LastSync;
                    LastSync = -1;
                    pcmSize = 0;
                    pcmL = realloc( pcmL, 0 );
                    pcmR = realloc( pcmR, 0 );
                    QueueEnqueueItem( WritingQueue, (QueueItem_t)frameAudio );
                }
            }
            QueueEnqueueItem( WritingQueue, (QueueItem_t)frame );
            break;
        default:
            nuv_free_frame( frame );
            break;
        }
    }

    decodeAudioClose();
    closeAudio();

    return( NULL );
}


/*
 * vim:ts=4:sw=4:ai:et:si:sts=4
 */
