/*
 *  This file is part of the nuvtools package
 *  Copyright (C) 2004, 2005 Gavin Hurlbut
 *
 *  nuvtools is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*HEADER---------------------------------------------------
* $Id: vidReencode.c 105 2005-04-24 03:10:16Z gjhurlbu $
*
* Copyright 2004, 2005 Gavin Hurlbut
* All rights reserved
*
*/

#include "environment.h"
#include <stdlib.h>
#include "format.h"
#include "nuvread.h"
#include "queue.h"
#include "linked_list.h"
#include "avcodec.h"
#include "minilzo.h"
#include "types.h"
#include "logging.h"

/* CVS generated ID string */
static char ident[] _UNUSED_ = 
    "$Id: vidReencode.c 105 2005-04-24 03:10:16Z gjhurlbu $";

extern QueueObject_t *VideoInQueue;
extern QueueObject_t *WritingQueue;
extern bool GlobalAbort;


static nuvFile        *nv;
static nuvFrame       *frame;

nuvFrame *decodeRaw( nuvFile *nv, nuvFrame *frameIn );
nuvFrame *decodeRtjpeg( nuvFile *nv, nuvFrame *frameIn );
nuvFrame *decodeRawLzo( nuvFile *nv, nuvFrame *frameIn );
nuvFrame *decodeRtjpegLzo( nuvFile *nv, nuvFrame *frameIn );
nuvFrame *decodeMjpeg( nuvFile *nv, nuvFrame *frameIn );
nuvFrame *decodeMpeg4( nuvFile *nv, nuvFrame *frameIn );
void decodeRtjpegInit( nuvFile *nv );
int findDecode( fourccDef_t *fourCCTable, int count, unsigned char frameType,
                unsigned int fourcc );
nuvFrame *encodeVideo( nuvFile *nv, nuvFrame *frameIn );
void closeVideo( void );
void writeFrameToDisk(nuvFrame *frame, int count);


static fourccDef_t fourCC[] = {
    { '0', { .text = "RJPG" }, decodeRaw },
    { '1', { .text = "RJPG" }, decodeRtjpeg },
    { '2', { .text = "RJPG" }, decodeRtjpegLzo },
    { '3', { .text = "RJPG" }, decodeRawLzo },
    { '4', { .text = "DIVX" }, decodeMpeg4 },
    { '4', { .text = "MJPG" }, decodeMjpeg }
};
static int fourCC_count = NELEMENTS(fourCC);

void *VideoReencodeThread( void *arg )
{
    int             done;
    int             decodeIndex;
    int             ret;
    int             count=0;

    done = FALSE;

    ret = lzo_init();
    if( ret < 0 ) {
        LogPrint( LOG_CRIT, "Couldn't initialize LZO!\n" );
    }

    while( !done ) {
        frame = (nuvFrame *)QueueDequeueItem( VideoInQueue, -1 );
        if( !frame || GlobalAbort ) {
            return( NULL );
        }

        switch( frame->header.frametype ) {
        case 'I':
            nv = (nuvFile *)malloc( sizeof(nuvFile) );
            if( !nv ) {
                LogPrint( LOG_CRIT, "Couldn't copy the file info in "
                                    "vidReencode\n" );
                return( NULL );
            }
            memcpy( nv, frame->data, frame->length );

            if( memcmp( &nv->video_4cc, "RJPG", 4 ) == 0 ) {
                decodeRtjpegInit(nv);
                memcpy( &nv->out_video_4cc, "XVID", 4 );
            }

            QueueEnqueueItem( WritingQueue, (QueueItem_t)frame );
            break;
        case 'E':
            QueueEnqueueItem( WritingQueue, (QueueItem_t)frame );
            done = TRUE;
            break;
        case 'V':
            count++;
            decodeIndex = findDecode( fourCC, fourCC_count, 
                                      frame->header.comptype, nv->video_4cc );
            if( decodeIndex != -1 ) {
                frame = fourCC[decodeIndex].decode(nv, frame);
                if( frame ) {
                    if( frame->needsEncode ) {
                        /* if we need to recode, do it here */
#if 0
                        writeFrameToDisk(frame, count);
#endif
                        frame = encodeVideo(nv, frame);
                    }

                    if( frame ) {
                        QueueEnqueueItem( WritingQueue, (QueueItem_t)frame );
                    }
                }
            }
            break;
        case 'S':
            QueueEnqueueItem( WritingQueue, (QueueItem_t)frame );
            break;
        default:
            nuv_free_frame( frame );
            break;
        }
    }

    closeVideo();

    return( NULL );
}


/*
 * vim:ts=4:sw=4:ai:et:si:sts=4
 */
